@@ -52,3 +52,6 @@ to reside. This is set automatically from sysconfdir by ./configure .*/
 
 /*Define if have the misfortune to be running Solaris*/
 #undef solaris
+
+/*Define if running on digital.*/
+#undef BROKEN_TERMIOS_VMIN
