@@ -1,6 +1,6 @@
 /*
  *  Queue load balancing system
- *  $Revision: 1.1 $
+ *  $Revision: 1.4 $
  *
  *  Copyright (C) 1998, 1999 Werner G. Krebs
  *
@@ -244,14 +244,14 @@ case 'l': {
  break;
 }
  case 'h': {
+   if (!optarg || !*optarg) break;
    if (!strcmp(optarg,"+")) break; /* '+' is wildcard; ignore.*/
    if (strlen(optarg) > MAXHOSTNAMELEN) {
      fprintf(stderr, "Queue: hostname too long.\n");
      exit(1);
    }
-   prefhost = canonicalhost(optarg);
-   if(!validhost(prefhost)) {Usage(); exit(1);}
-   prefhost = localhost(prefhost);
+   prefhost = strdup(canonicalhost(optarg));
+   if(!validhost(prefhost,0)) {fprintf(stderr, "Queue: invalid queue hostname: %s\n",prefhost); exit(1);}
    onlyhost = prefhost;
    break;}
  case 'H': {
@@ -260,9 +260,9 @@ case 'l': {
      fprintf(stderr, "Queue: hostname too long.\n");
      exit(1);
    }
-   prefhost = canonicalhost(optarg);
-   if(!validhost(prefhost,0)) {Usage(); exit(1);}
-   prefhost=localhost(prefhost);
+   prefhost = strdup(canonicalhost(optarg));
+   if(!validhost(prefhost,0)) {fprintf(stderr, "Queue: invalid queue hostname: %s\n",prefhost); exit(1);}
+   prefhost=prefhost;
    onlyhost = NULL;
    break;}
 case 'd': {
@@ -334,9 +334,8 @@ else{
      fprintf(stderr, "Queue: hostname too long.\n");
      exit(1);
    }
-   prefhost = canonicalhost(argv[optind++]);
-   if(!validhost(prefhost)) {Qshusage(); exit(1);}
-   prefhost = localhost(prefhost);
+   prefhost = strdup(canonicalhost(argv[optind++]));
+   if(!validhost(prefhost,0)) {Qshusage(); exit(1);}
    onlyhost = prefhost;
 }
 }
@@ -576,7 +575,9 @@ name1 = cfname;
 
 /*If file exists, goto another.*/
 
-if((fd=open(name,O_WRONLY|O_CREAT|O_EXCL))<0) {
+/*WGK 99/3/4 added S_IRUSR|S_IWUSR based on suggestion from zblazek@engr.UVic.CA*/
+
+if((fd=open(name,O_WRONLY|O_CREAT|O_EXCL,S_IRUSR|S_IWUSR))<0) {
 if(errno==EEXIST) goto another; 
   perror("open");
 exit(2);
@@ -600,7 +601,7 @@ mylock.l_type = F_WRLCK;
 mylock.l_whence = SEEK_SET;
 mylock.l_start = 0;
 mylock.l_len = 0;
-if(fcntl(fd,F_SETLKW,&mylock)<0) {
+if(fcntl(fd,F_SETLKW,&mylock)<0 && errno != ENOLCK) {
   perror("fcntl");
   exit(2);
 } 
@@ -999,7 +1000,7 @@ in the style of TCP/IP wrapper.	 */
 	} else
 	hostname = "unresolved"; /*Our DNS must be down and this will probably fail.*/
 
-if(!validhost(hostname,socket)) exit(1);
+if(!validhost(hostname,newsocket)) exit(1);
 
 /*A null byte to queued tells it that everythig is OK.*/
 
@@ -1045,7 +1046,11 @@ if(ttyinput) {
      myterm.c_iflag &= ~(IXON|IXANY|IXOFF|ICRNL|IGNCR|INLCR|IUCLC);
      myterm.c_lflag &= ~(ICANON|ISIG|ECHO|ECHOE|ECHOK|ECHONL|IEXTEN|XCASE);
 if(ttyoutput) myterm.c_oflag &= ~OPOST;
+#ifndef BROKEN_TERMIOS_VMIN
      myterm.c_cc[VMIN]=2;
+#else
+     myterm.c_cc[VMIN]=1;
+#endif
      myterm.c_cc[VTIME]=1;
      tcsetattr(0,0,&myterm);
    }
@@ -1437,8 +1442,7 @@ if(ReadHosts(QHOSTSFILE) < 0)
   }
  }
 
- 
- for (i=0;i<NHosts;++i) if (strcmp(hostname,Hosts[i].host)==0) flag = 0;
+ for (i=0;i<NHosts;++i) if (strcasecmp(hostname,Hosts[i].host)==0) flag = 0;
  if (flag) {
 
 		fprintf(stderr,"Host %s is not a valid server host.\n",hostname);
